;;;  WobblyPline.LSP [Command name: WPL]
;;;  To draw a Polyline that wobbles "randomly" from the plain path created or selected.
;;;  Options:
;;;    1.  create new, or select EXisting, path of any appropriate type;
;;;    2.  number of line segments in resulting polyline;
;;;    3.  degree of randomization [as a percentage of average segment length];
;;;    4.  after first use, re-use PRevious path [changing any or all choices], or
;;;    5.  after first use, Redo wholesale, using previous path and all previous choices;
;;;    6.  Retain or Delete base path.
;;;  Under Existing-path option, asks User to select again if nothing selected, or selected object
;;;    is an inappropriate entity type, or is on a locked Layer under Delete-path option.
;;;  Draws 3DPolyline [randomized only in X-Y direction] if path is one; otherwise draws 2D
;;;    ["lightweight"] Pline.
;;;  Draws on current Layer, unless Deleting Existing path; if so, draws on its Layer, assuming
;;;    the intent is to replace it with the wobbly-ized version.
;;;  Remembers choices and offers them as defaults for subsequent use.
;;;
;;;  Can also be used:
;;;    A.  in un-randomized way [e.g. to turn Circle into regular polygon, or subdivide Line
;;;      into equal Pline-segment pieces], by specifying 0 percent displacement of points;
;;;    B.  to straighten any open path into single-line-segment Pline, by specifying 1 segment.
;;;
;;;  Kent Cooper, last edited 6 October 2022
;
(defun C:WPL (/ *error* cmde osm typetemp pathsel deltemp 3Dpath segtemp pcttemp
  segavg maxdis intpt blips base angdisp distdisp)
;
  (defun *error* (errmsg)
    (if (not (wcmatch errmsg "Function cancelled,quit / exit abort,console break"))
      (princ (strcat "\nError: " errmsg))
    ); end if
    (setvar 'osmode osm)
    (setvar 'blipmode blips)
    (command-s "_.undo" "_end")
    (setvar 'cmdecho cmde)
  ); end defun *error*
;
  (vl-load-com)
  (setq cmde (getvar 'cmdecho))
  (setvar 'cmdecho 0)
  (command "_.undo" "_begin")
  (setq
    blips (getvar 'blipmode)
    osm (getvar 'osmode)
  ); end setq
;
  (if *wpl*
    (initget "Line Arc Circle Pline 3dpoly Ellipse Spline EXisting PRevious Redo")
    (initget "Line Arc Circle Pline 3dpoly Ellipse Spline EXisting"); no PR/R options on first use
  ); end if
  (setq typetemp
    (getkword
      (strcat
        "\nPath type [{new} Line(single)/Arc/Circle/Pline(2D)/3dpoly/Spline/Ellipse/EXisting"
        (if *wpl*
          "/PRevious/Redo"
          ""
        ); end if
        "] <"
        (if *wpl* *wpltype* "Pline"); at first use, Polyline default; otherwise, previous type
        ">: "
      ); end strcat
    ); end getkword
    *wpltype*
      (cond
        (typetemp); if User typed something other than Enter, use it
        (*wpltype*); if Enter and there's a previous type, use that
        (T "Pline"); otherwise [Enter on first use], Polyline default
      ); end cond & *wpltype*
  ); end setq
;
  (if (= *wpltype* "PRevious") (entdel *wpl*)); if re-using Previous path with option
    ;  of new choices [but not wholesale Redo], delete previous wobbly Polyline
  (if (and (= *wpltype* "PRevious") (= *wpldel* "Delete")) (entdel *wplpath*)); bring back Previous path
;
  (if (/= *wpltype* "Redo"); if not Redoing,
    (progn
      (initget "Retain Delete")
      (setq
        deltemp
          (getkword
            (strcat
              "\nRetain or Delete base path [R/D] <"
              (if *wpl* (substr *wpldel* 1 1) "D"); at first use, Delete default; otherwise, previous type
              ">: "
            ); end strcat
          ); end getkword
        *wpldel*
          (cond
            (deltemp); if User typed something, use it
            (*wpldel*); if Enter and there's a previous choice, use that
            (T "Delete"); otherwise [Enter on first use], Delete
          ); end cond & *wpldel*
      ); end setq
    ); end progn
  ); end if
;
  (cond ; select or make path
    ((and (= *wpltype* "EXisting") (= *wpldel* "Delete")); check selected object for locked Layer if Delete option
      (prompt "\nTo make an EXisting path wobbly,")
      (while
        (not
          (and
            (setq pathsel (ssget "_:S" '((0 . "LINE,ARC,CIRCLE,*POLYLINE,ELLIPSE,SPLINE"))))
            (= (logand (cdr (assoc 70 (tblsearch "layer" (cdr (assoc 8 (entget (ssname pathsel 0))))))) 4) 0); 0 for Unlocked, 4 for Locked
          ); end and
        ); end not
        (prompt "\nNothing selected, or object is not a finite path type, or is on a Locked Layer; try again:")
      ); end while
    ); end first condition - EXisting with Delete
    ((= *wpltype* "EXisting")
      (prompt "\nTo make an EXisting path wobbly,")
      (while
        (not (setq pathsel (ssget "_:S" '((0 . "LINE,ARC,CIRCLE,*POLYLINE,ELLIPSE,SPLINE")))))
        (prompt "\nNothing selected, or object is not a finite path type; try again:")
      ); end while
    ); end second condition - EXisting with Retain
    ((and (= *wpltype* "Redo") (= *wpldel* "Delete")) (entdel *wplpath*)); bring back path
    ((= *wpltype* "Line") (setvar 'cmdecho 1) (command "_.line" pause pause "") (setvar 'cmdecho 0))
    ((not (wcmatch *wpltype* "Redo,PRevious")); all other entity types
      (setvar 'cmdecho 1)
      (command (strcat "_." *wpltype*))
      (while (> (getvar 'cmdactive) 0) (command pause))
      (setvar 'cmdecho 0)
    ); end fourth condition
  ); end cond
;
  (setq
    *wplpath* ; set object as base path
      (cond
        ((= *wpltype* "EXisting") (ssname pathsel 0))
        ((or (= *wpltype* "Redo") (= *wpltype* "PRevious")) *wplpath*)
        ((entlast)); newly created path
      ); end cond & *wplpath*
    pathlength (vlax-curve-getDistAtParam *wplpath* (vlax-curve-getEndParam *wplpath*))
    3Dpath (= (cdr (assoc 0 (entget *wplpath*))) "POLYLINE")
  ); end setq
;
  (if (/= *wpltype* "Redo"); if not Redoing,
    (progn ; then - ask for other input
;
      (if *wpl* (initget 6) (initget 7)); no Enter on first use, no 0, no negative
      (while
        (and
          (setq
            segtemp
              (getint
                (strcat
                  "\nNumber of segments"
                  (if *wplseg* (strcat " <" (itoa *wplseg*) ">") ""); no default on first use
                  ": "
                ); end strcat
              ); end getint & segtemp
            *wplseg* (if segtemp segtemp *wplseg*)
          ); end setq
          (vlax-curve-isClosed *wplpath*); with next line, prevent 1 segment on closed path
          (< *wplseg* 2)
        ); end and
        (prompt "\nMin. 2 {recommend min. 3} segments for closed path;")
        (if *wpl* (initget 6) (initget 7))
      ); end while
;
      (if (> *wplseg* 1); don't ask displacement if straightening
        (progn
          (if *wplpct* (initget 4) (initget 5)); no Enter on first use, 0 allowed, no negative
          (setq
            pcttemp
              (getreal
                (strcat
                  "\nMaximum displacement as percentage of average segment length"
                  (if *wplpct* (strcat " <" (rtos *wplpct* 2 2) ">") ""); no default on first use
                  ": "
                ); end strcat
              ); end getreal
            *wplpct* (if pcttemp pcttemp *wplpct*)
          ); end setq
        ); end progn
      ); end if
;
    ); end progn - not Redoing
    (entdel *wpl*); else - if Redoing, eliminate previous one
  ); end if
;
  (setq
    segavg (/ pathlength *wplseg*)
    maxdis (* segavg *wplpct* 0.01); maximum displacement
    intpt 1 ; starting value for intermediate point
  ); end setq
  (setvar 'blipmode 0)
  (setvar 'osmode 0)
;
  (if (not 3Dpath) (command "_.ucs" "_new" "_object" *wplpath*)) ; set UCS to match object if appropriate
  (if (= *wpldel* "Delete") (setvar 'clayer (cdr (assoc 8 (entget *wplpath*))))) ; if Deleting Existing path, draw on same Layer
;
  (command
    (if 3Dpath "_.3dpoly" "_.pline"); command type - Pline for all but 3D Polylines
    (trans (vlax-curve-getStartPoint *wplpath*) 0 1)
  ); end command [leaving in Pline/3dpoly]
  (while (< intpt *wplseg*)
    (setq
      base (vlax-curve-getPointAtDist *wplpath* (* segavg intpt)); un-randomized intermediate location
      angdisp (* 2 pi (/ (atoi (substr (rtos (rem (getvar 'cdate) 1) 2 16) (- 17 (fix (rem (cadr base) 3)) (fix (abs (* (sin intpt) 10)))) 2)) 100.0))
        ; randomized angle
      distdisp (* maxdis (/ (atoi (substr (rtos (rem (getvar 'date) 1) 2 16) (- 18 (rem (fix (sqrt (abs (car base)))) 5) (rem intpt 3)) 2)) 100.0))
        ; randomized distance
      intpt (1+ intpt); increment intermediate point number
    ); end setq
    (command (polar (trans base 0 1) angdisp distdisp)); feed randomized location out to Pline/3dpoly command
  ); end while
  (if (vlax-curve-isClosed *wplpath*)
    (command "_close")
    (command (trans (vlax-curve-getEndPoint *wplpath*) 0 1) ""); last point and end for open path
  ); end if
;
  (if (not 3Dpath) (command "_.ucs" "_prev")); reset UCS if appropriate
  (if (and (member *wpltype* '("EXisting" "PRevious" "Redo")) (= *wpldel* "Delete")) (command "_.layerp"))
    ; reset Layer if appropriate
;
  (if (= *wpldel* "Delete") (entdel *wplpath*))
  (setq *wpl* (entlast))
  (setvar 'osmode osm)
  (setvar 'blipmode blips)
  (command "_.undo" "_end")
  (setvar 'cmdecho cmde)
  (princ)
); end defun
(prompt "Type WPL to make a Wobbly PolyLine.")
